clear all;
close all;

w=5e-6; %grain width
h=5e-6; %grain height
nx=8; %# of x grid (need to be even number)
ny=4; %# of y grid
delt=1; %unit time

%properties
g1=1.12; %free energy of vapor phase
g2=1.12; %free energy of solid phase

%generate grid
[coord0, element]=GridGen(w,h,nx,ny);
COORD(1,:,:)=coord0(:,:);
%boundary node: left=1, center: nx/2+1, right=nx+1
%surface element:1~nx, grain boundary element from (nx+1)
    
%plot initial configuration
Nnode=size(coord0,1); %# of elements
Nel=size(element,1); %# of elements
figure
for i=1:length(element)
    node1=element(i,1); %node #
    node2=element(i,2); %node #
    x=[coord0(node1,1),coord0(node2,1)];
    y=[coord0(node1,2),coord0(node2,2)];
    plot(x,y,'x-');hold on;
end
axis([-2.5e-6,2.5e-6,0,5.5e-6]);
%hold off;

%Boundaries
Bindex=[1,3*nx/2+1,3*nx+1,3*nx+4,3*nx+5,3*nx+6,3*Nnode+nx+1:3*Nnode+Nel];
%x(left), x(center), x(right), x(base), y(base), J(base), Jm at grain
%boundaries

%Temperature profile
T0=300;
Th=400;
for i=1:60*5
    Tprofile(i)=T0+(i-1)*(Th-T0)/(60*5);
end
for i=60*5+1:60*25
    Tprofile(i)=Th;
end
for i=60*25+1:60*30
    Tprofile(i)=Th-(i-60*25)*(Th-T0)/(60*5);
end
%plot(Tprofile);
    
%Euler method
for n=1:60*30
    coord(:,:)=COORD(n,:,:);
    t=(n-1)*delt; %time
    T=Tprofile(n); %temperature
    
    %material properties
    [L,M,gamma]=Mproperty(T);
    gammas=gamma;
    gammab=gamma;
    
    Nnode=size(coord,1); %# of elements
    Nel=size(element,1); %# of elements
    
    %weak form
    Hmat=zeros(3*Nnode+Nel,3*Nnode+Nel); %global H matrix
    fmat=zeros(3*Nnode+Nel,1); %global f matrix
    for i=1:Nel
        node1=element(i,1); %node #
        node2=element(i,2); %node #
        Node1=coord(node1,:);
        Node2=coord(node2,:);
        t=Node2-Node1; %tangential vector
        l=norm(t); %length of the element
        SIN=t(2)/l; %sin(theta)
        COS=t(1)/l; %cos(theta)
        
        %local H matrix
        Hloc=Hlocal(L,M,l,SIN,COS);
        %assemble H matrix
        index=[(node1-1)*3+1:node1*3,(node2-1)*3+1:node2*3, Nnode*3+i];
        for j=1:7
            for k=1:7
                Hmat(index(j),index(k))=Hmat(index(j),index(k))+Hloc(j,k);
            end
        end
        
        %local f matrix
        if i<=nx
            gamma=gammas; %surface element
            dgamma=0;
        else
            gamma=gammab; %boundary element
            dgamma=0;
            g1=g2;
        end
        floc=flocal(gamma,dgamma,g1,g2,SIN,COS,l);
        %assemble f matrix
        index=[(node1-1)*3+1,(node1-1)*3+2,(node2-1)*3+1,(node2-1)*3+2];
        for j=1:4
            fmat(index(j))=fmat(index(j))+floc(j);
        end
    end
    
    %Remove the singularity
    smallx=1e-3/L*w/nx*1000;
    smally=1e-3/L*h/ny*1000;
    for i=1:Nnode
        Hmat(3*(i-1)+1,3*(i-1)+1)=Hmat(3*(i-1)+1,3*(i-1)+1)+smallx;
        Hmat(3*(i-1)+2,3*(i-1)+2)=Hmat(3*(i-1)+2,3*(i-1)+2)+smally;
    end
    
    %boundary condition    
    HmatTemp=Hmat;
    HmatTemp(Bindex,:)=[];
    HmatTemp(:,Bindex)=[];
    fmatTemp=fmat;
    fmatTemp(Bindex)=[];
    
    %solve
    utemp=inv(HmatTemp)*fmatTemp;
    u=zeros(3*Nnode+Nel,1);
    uindex=[1:3*Nnode+Nel];
    uindex=setdiff(uindex,Bindex);
    for i=1:length(uindex)
        u(uindex(i))=utemp(i);
    end
    
    %update new node
    for i=1: Nnode
        coord(i,1)=coord(i,1)+delt*u((i-1)*3+1);
        coord(i,2)=coord(i,2)+delt*u((i-1)*3+2);
    end
   
    coordnew=NodeUpdate(coord,w,h,nx,ny);
    COORD(n+1,:,:)=coordnew;
end
    
%plot updated configuration
%figure
for i=1:length(element)
    node1=element(i,1); %node #
    node2=element(i,2); %node #
    x=[coordnew(node1,1),coordnew(node2,1)];
    y=[coordnew(node1,2),coordnew(node2,2)];
    plot(x,y,'rx-');hold on;
end
hold off;
%save Initial.mat coordnew element

